<?php

namespace Wincher;

class Plugin
{
    const SLUG = 'wincher';
    const VERSION = '2.1.1';

    /**
     * @var string
     */
    protected $slug;

    /**
     * @var WincherClient
     */
    protected $wincherClient;

    /**
     * Plugin constructor.
     */
    public function __construct()
    {
        add_action('admin_menu', [$this, 'addAdminMenu']);
        add_action('admin_enqueue_scripts', [$this, 'enqueueGlobalAssets']);
        add_action('rest_api_init', [$this, 'registerApiRoutes']);

        $key = get_option('wincher_api_key', null);

        $this->wincherClient = new WincherClient(
            $key,
            get_option('wincher_partner_id', null)
        );

        if (!$key) {
            add_action('pre_current_active_plugins', [$this, 'showActivateButton']);
        }
    }

    /**
     * @return WincherClient
     */
    public function getWincherClient()
    {
        return $this->wincherClient;
    }

    /**
     * Fired when the plugin activates.
     */
    public function activate()
    {
    }

    /**
     * Fired when the plugin deactivates.
     */
    public function deactivate()
    {
        delete_option('wincher_api_key');
        delete_option('wincher_account_type');
    }

    public function enqueueGlobalAssets()
    {
        wp_enqueue_style(self::SLUG . 'global', WINCHER_PLUGIN_BASE_URL . 'assets/css/global.css', [], self::VERSION);
        wp_enqueue_script(self::SLUG . 'global', WINCHER_PLUGIN_BASE_URL . 'assets/js/global.js', [], self::VERSION);
        wp_localize_script(self::SLUG . 'global', 'wincherConfig', [
            'apiNonce' => wp_create_nonce('wp_rest'),
            'apiBaseUrl' => rest_url($this->getApiNamespace()),
        ]);
    }

    public function addAdminMenu()
    {
        add_menu_page(
            'Google Rank Tracker by Wincher',
            'Google Rankings',
            'manage_options',
            self::SLUG,
            [$this, 'displayOverview'],
            $this->getImageBase64(WINCHER_PLUGIN_BASE_PATH . 'assets/img/mark.svg'),
            '2.34'
        );

        add_submenu_page(
            self::SLUG,
            'Google Rank Tracker by Wincher',
            __('Dashboard', self::SLUG),
            'manage_options',
            self::SLUG,
            [$this, 'displayOverview']
        );

        if (get_option('wincher_account_type') === 'FREE') {
            global $submenu;
            $upgradeText = __('Subscribe now', self::SLUG);
            $submenu[self::SLUG][] = [
                "<span class='wincher-upgrade-link'>$upgradeText</span>",
                'manage_options',
                'https://www.wincher.com/subscribe?referer=wordpress&utm_source=wordpress&utm_medium=link&utm_content=submenu'
            ];
        }
    }

    public function showActivateButton()
    {
        ?>
            <div class="wincher-activate">
                <span>Don't forget to activate Wincher to keep track of your Google rankings!</span>
                <a href="admin.php?page=<?php echo self::SLUG; ?>">Start using Wincher now</a>
            </div>
        <?php
    }

    public function displayOverview()
    {
        $page = new DashboardPage($this);
        $page->render();
    }

    public function registerApiRoutes()
    {
        $routes = [
            [
                'route' => 'token',
                'methods' => \WP_REST_Server::CREATABLE,
                'action' => 'AuthController::token'
            ],
            [
                'route' => 'signup',
                'methods' => \WP_REST_Server::CREATABLE,
                'action' => 'AuthController::signup'
            ],
            [
                'route' => 'status',
                'methods' => \WP_REST_Server::READABLE,
                'action' => 'StatusController::get'
            ],
            [
                'route' => 'search-engines',
                'methods' => \WP_REST_Server::READABLE,
                'action' => 'DashboardController::getSearchEngines'
            ],
            [
                'route' => 'dashboard',
                'methods' => \WP_REST_Server::READABLE,
                'action' => 'DashboardController::getDashboardData'
            ],
            [
                'route' => 'ranking',
                'methods' => \WP_REST_Server::CREATABLE,
                'action' => 'DashboardController::getRankingHistory'
            ],
            [
                'route' => 'keywords',
                'methods' => \WP_REST_Server::CREATABLE,
                'action' => 'DashboardController::createKeyword'
            ],
            [
                'route' => 'keywords/batch-delete',
                'methods' => \WP_REST_Server::CREATABLE,
                'action' => 'DashboardController::deleteKeywords'
            ]
        ];

        $namespace = $this->getApiNamespace();

        foreach ($routes as $opts) {
            list($class, $callbackMethod) = explode('::', $opts['action']);
            $class = '\\Wincher\\Controller\\' . $class;
            $controller = new $class($this->getWincherClient());

            register_rest_route($namespace, $opts['route'], [
                'methods' => $opts['methods'],
                'callback' => [$controller, $callbackMethod],
                'permission_callback' => [$controller, 'hasPermission']
            ]);
        }
    }

    private function getApiNamespace()
    {
        return self::SLUG . '/v' . self::VERSION;
    }

    private function getImageBase64($path)
    {
        $data = base64_encode(file_get_contents($path));
        return 'data:image/svg+xml;base64,' . $data;
    }
}
