<?php

namespace Wincher;

use GuzzleHttp\Client;
use GuzzleHttp\Exception\ClientException;
use GuzzleHttp\Exception\GuzzleException;

class WincherClient
{
    const BASE_URI = 'https://api.wincher.com/v0/';

    /**
     * @var Client
     */
    protected $client;

    /**
     * @var string
     */
    protected $accessToken;

    /**
     * @var string
     */
    protected $partnerId;

    /**
     * WincherClient constructor.
     *
     * @param string|null $accessToken
     * @param string|null $partnerId
     */
    public function __construct($accessToken=null, $partnerId=null)
    {
        $this->accessToken = $accessToken;
        $this->partnerId = $partnerId;

        $this->client = new Client([
            'base_uri' => static::BASE_URI
        ]);
    }

    /**
     * Returns all currently available search engines.
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function getSearchEngines()
    {
        return $this->request('GET', 'search-engines');
    }

    /**
     * Returns information about the authenticated account.
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function getAccount()
    {
        return $this->request('GET', 'account');
    }

    /**
     * Returns all account domains.
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function getDomains()
    {
        return $this->request('GET', 'domains');
    }

    /**
     * Signs up a new Wincher WordPress user.
     *
     * @param string $email
     * @param string $firstName
     * @param string $lastName
     * @param string $password
     * @param array $searchEngine
     * @param string $domain
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function signup($email, $firstName, $lastName, $password, array $searchEngine, $domain)
    {
        return $this->request('POST', 'signup/wordpress', [
            'json' => [
                'email' => $email,
                'first_name' => $firstName,
                'last_name' => $lastName,
                'password' => $password,
                'search_engine' => $searchEngine,
                'domain' => $domain
            ]
        ]);
    }

    /**
     * Get an API authentication token that can be used for subsequent API calls.
     *
     * @param string $email
     * @param string $password
     * @return array
     *
     * @throws GuzzleException
     */
    public function token($email, $password)
    {
        return $this->request('GET', 'wordpress/token', [
            'headers' => [
                'Authorization' => 'Basic ' . base64_encode("{$email}:$password")
            ]
        ]);
    }

    /**
     * Returns all data used by the WordPress dashboard.
     *
     * @param int $domainId
     * @param \DateTime|null $startDate
     * @param \DateTime|null $endDate
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function getDashboardData($domainId, \DateTime $startDate=null, \DateTime $endDate=null)
    {
        if (!$startDate) {
            $startDate = new \DateTime('-30 days');
        }

        if (!$endDate) {
            $endDate = new \DateTime();
        }

        return $this->request('GET', 'wordpress/dashboard', [
            'query' => [
                'id' => $domainId,
                'start_date' => $startDate->format('Y-m-d'),
                'end_date' => $endDate->format('Y-m-d')
            ]
        ]);
    }

    /**
     * Returns ranking history for one or more specific keywords.
     *
     * @param array $keywordIds
     * @param \DateTime|null $startDate
     * @param \DateTime|null $endDate
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function getRankingHistory(array $keywordIds, \DateTime $startDate=null, \DateTime $endDate=null)
    {
        if (!$startDate) {
            $startDate = new \DateTime('-30 days');
        }

        if (!$endDate) {
            $endDate = new \DateTime();
        }

        return $this->request('POST', 'ranking-history', [
            'json' => [
                'keyword_ids' => $keywordIds,
                'start_date' => $startDate->format('Y-m-d'),
                'end_date' => $endDate->format('Y-m-d')
            ]
        ]);
    }

    /**
     * Creates a new keyword for a domain.
     *
     * @param int $domainId
     * @param string $keyword
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function createKeyword($domainId, $keyword)
    {
        return $this->request('POST', "domains/$domainId/keywords", [
            'json' => ['keyword' => $keyword]
        ]);
    }

    /**
     * Deletes keywords from a domain.
     *
     * @param int $domainId
     * @param array $keywordIds
     *
     * @return array
     *
     * @throws GuzzleException
     */
    public function deleteKeywords($domainId, array $keywordIds)
    {
        return $this->request('POST', "domains/$domainId/keywords/batch-delete", [
            'json' => ['ids' => $keywordIds]
        ]);
    }

    /**
     * Performs a request against the API.
     *
     * @param $method
     * @param $path
     * @param array $options
     * @return array
     *
     * @throws GuzzleException
     */
    protected function request($method, $path, array $options=[])
    {
        if (!isset($options['timeout'])) {
            $options['timeout'] = 10;
        }

        if (!isset($options['headers'])) {
            $options['headers'] = [];
        }

        if (!isset($options['headers']['Authorization'])) {
            if ($this->accessToken) {
                $options['headers']['Authorization'] = 'Token ' . $this->accessToken;
            }
        }

        if ($this->partnerId) {
            $options['headers']['X-Partner-Id'] = $this->partnerId;
        }

        try {
            $resp = $this->client->request($method, $path, $options);
        } catch (ClientException $e) {
            $resp = $e->getResponse();
        }

        $data = json_decode($resp->getBody(), true);

        return [
            'status' => $resp->getStatusCode(),
            'data' => $data
        ];
    }
}
